.spacer {
    margin-top: 20px;
    margin-bottom: 20px;
}
.span6 table {
    table-layout:fixed;
    word-wrap: break-word;
}
.span6 table caption + tbody tr td:first-child {
    width: 25%;
}
.btn-group-justified{
    display: inline-block;
    width: 100%;
    float: left;
}
.btn-group-justified .btn{
    display: table-cell !important;
    float: none;
    width: 1%;
}
.panel {
  padding: 15px;
  margin-top: 60px;
  margin-bottom: 20px;
  background-color: #ffffff;
  border: 1px solid #dddddd;
  border-radius: 4px;
  -webkit-box-shadow: 0 1px 1px rgba(0, 0, 0, 0.05);
  box-shadow: 0 1px 1px rgba(0, 0, 0, 0.05);
}
.panel-heading {
  padding: 10px 15px;
  margin: -15px -15px 15px;
  font-size: 17.5px;
  font-weight: 500;
  background-color: #f5f5f5;
  border-bottom: 1px solid #dddddd;
  border-top-right-radius: 3px;
  border-top-left-radius: 3px;
}
.panel-footer {
  padding: 10px 15px;
  margin: 15px -15px -15px;
  background-color: #f5f5f5;
  border-top: 1px solid #dddddd;
  border-bottom-right-radius: 3px;
  border-bottom-left-radius: 3px;
}
.panel-primary {
  border-color: #428bca;
}
.panel-primary .panel-heading {
  color: #ffffff;
  background-color: #428bca;
  border-color: #428bca;
}
.panel-success {
  border-color: #d6e9c6;
}
.panel-success .panel-heading {
  color: #468847;
  background-color: #dff0d8;
  border-color: #d6e9c6;
}
.panel-warning {
  border-color: #fbeed5;
}
.panel-warning .panel-heading {
  color: #c09853;
  background-color: #fcf8e3;
  border-color: #fbeed5;
}
.panel-danger {
  border-color: #eed3d7;
}
.panel-danger .panel-heading {
  color: #b94a48;
  background-color: #f2dede;
  border-color: #eed3d7;
}
.panel-info {
  border-color: #bce8f1;
}
.panel-info .panel-heading {
  color: #3a87ad;
  background-color: #d9edf7;
  border-color: #bce8f1;
}
.panel>.table, .panel>.table-responsive>.table {
      margin-bottom: 0;
}

.filter-column {
  width: 50%;
}

.filter-column.left {
  float: left;
}

.filter-column.right{
  float: right;
}

.filter-actions {
  width: 100%;
  display: inline-block;
}

/* Modify the bootstrap defaults to better discern red and green */
.label-important,
.badge-important {
    background-color: #db120d;
}
.label-success,
.badge-success {
    background-color: #2ec730;
}

/* Modify bootstrap dropdown menu to have white font colour instead of
 * gray (recall that the background is green) */
.nav-collapse .nav > li > a,
.nav-collapse .dropdown-menu a {
  color: #ffffff;
}
