{% extends "base.html" %}

{% block container %}
    <div class="span12">
        <p>
            {% if message %}
                {{ message }}
            {% else %}
                Error, page not found
            {% end %}
        </p>
    </div>
{% end %}
